@@ -5,6 +5,8 @@ __author__ = 'Alberto Martín Mateos and Niloufar Shoeibi'
 __version__ = '0.0.1'
 
 from .api import load_file
+from .api import duplicates_detection
+from .api import close_duplicates
 from .api import merge_dataframes
 from .api import profile_connections
 from .api import building_node_list
