@@ -628,41 +628,50 @@ def time_series_tw (df):
     return df 
 
 #-------------------------------------------Ratios and other advanced features----------------------------------#
-def duplicates_detection(data, i):
-    if i < len(data):
-        possible_duplicated = []
-        for j in range(len(data)):
-            if j !=i:
-                list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
-                list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
-                distance, path = fastdtw(list_dtw_1,list_dtw_2)
-                if (distance < 600):
-                    possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
-        if (len(possible_duplicated)==0):
-            possible_duplicated.append({"screen_name": "Empty", "distance": "Empty", "tw_similarity": 0})   
-            data["possible_duplicates"].iloc[i] = possible_duplicated        
-        else: 
-            data["possible_duplicates"].iloc[i] = possible_duplicated          
-            for t in range(len(data["possible_duplicates"].iloc[i])):
-                #text_1 is the differences possible duplicates with the profile evaluate
-                text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
-                text_1 = pd.DataFrame()
-                for a in range(len(text_1_aux)):
-                    aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
-                    text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
-                text_1 = list(itertools.chain(*text_1["text"]))                
-                text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))
-                if "Empty" in text_1:
-                  text_1 = list(filter(("Empty").__ne__, text_1))
-                if "Empty" in text_2:  
-                  text_2 = list(filter(("Empty").__ne__, text_2))
-                comparacion = [item for item in text_1 if item in text_2]
-                if len(comparacion) > 0:
-                    data["possible_duplicates"].iloc[i][t]["tw_similarity"] = len(comparacion)/(data["num_userTw"].iloc[i])
-                else:
-                    data["possible_duplicates"].iloc[i][t]["tw_similarity"] = 0
+def duplicates_detection(data):
+    data["possible_duplicates"] = 0
+    for i in range(len(data)):
+        if i < len(data):
+            possible_duplicated = []
+            for j in range(len(data)):
+                if j !=i:
+                    list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
+                    list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
+                    distance, path = fastdtw(list_dtw_1,list_dtw_2)
+                    if (distance < 600):
+                        possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
+            if (len(possible_duplicated)==0):
+                possible_duplicated.append({"screen_name": "Empty", "distance": "Empty", "tw_similarity": 0,"tw_match_tw": "Empty"})   
+                data["possible_duplicates"].iloc[i] = possible_duplicated        
+            else: 
+                data["possible_duplicates"].iloc[i] = possible_duplicated          
+                for t in range(len(data["possible_duplicates"].iloc[i])):
+                    #text_1 is the differences possible duplicates with the profile evaluate
+                    text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
+                    text_1 = pd.DataFrame()
+                    for a in range(len(text_1_aux)):
+                        aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
+                        text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
+                    text_1 = list(itertools.chain(*text_1["text"]))   
+                    text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))               
+                    if "Empty" in text_1:
+                      text_1 = list(filter(("Empty").__ne__, text_1))
+                    if "Empty" in text_2:  
+                      text_2 = list(filter(("Empty").__ne__, text_2))
+                    if "" in text_1:
+                      text_1 = list(filter(("").__ne__, text_1))
+                    if "" in text_2:  
+                      text_2 = list(filter(("").__ne__, text_2))  
+                    comparacion = [item for item in text_2 if item in text_1]
+                    if len(comparacion) > 0:
+                        data["possible_duplicates"].iloc[i][t]["tw_similarity"] = len(comparacion)/(data["num_userTw"].iloc[i])
+                        data["possible_duplicates"].iloc[i][t]["text_match_tw"] = comparacion
+                    else:
+                        data["possible_duplicates"].iloc[i][t]["tw_similarity"] = 0
+                        data["possible_duplicates"].iloc[i][t]["text_match_tw"] = "Empty"
     return data
 	
+	
 def advanced_features(data):
     
     #Posible bot
@@ -673,7 +682,6 @@ def advanced_features(data):
     data["follow_rate"] = 0
     data["max_tw_day"] = 0
     data["index_max_day_tw"]=0
-    data["num_duplicates_list"] = 0
     data["possible_duplicates"] = 0
 
     data["num_userTw"]= data["num_ownTw"] + data["RT"]
@@ -686,7 +694,7 @@ def advanced_features(data):
     data["fake_sum"] = data["default_profile_image"] + data["biography_profile"]+ data["screen_name_bot"]
     data["fake_rate"] = ((data["tweets_url"] + data["mentions"]) /data["num_ownTw"]).replace([np.inf, -np.inf], np.nan).fillna(0)
     data["follow_rate"][data["followers"]== 0] = np.median(data["follow_rate"])
-    num_duplicates_tweets_list = []
+    
     for i in range(len(data)):
         if (len(str(data.iloc[i]["screen_name"])) > 4):
             if((data.iloc[i]["screen_name"][-1].isdigit()) & (data.iloc[i]["screen_name"][-2].isdigit()) & 
@@ -712,11 +720,12 @@ def advanced_features(data):
         else:
             data["max_ret_tw"].iloc[i] = max((data["retweet_tweets_list"].iloc[i]))
           
-        data = duplicates_detection(data, i)
+        #data = duplicates_detection(data, i)
         data["max_tw_day"].iloc[i] =max((data["tw_per_day_list"].iloc[i]))
         data["days_with_tw"].iloc[i] = len(data["tw_per_day_list"].iloc[i])
         data["index_max_day_tw"].iloc[i] = data["tw_per_day_list"].iloc[i].index(data["max_tw_day"].iloc[i])        
     data = trend_tweets_features(data) 
+    #data = close_duplicates(data)
         
     return data
 	
@@ -843,7 +852,7 @@ def abnormal_categories_profiles(data_join):
 
     #6.Possible new account with a high activity last days
     data_join["evaluate_NLP"][(data_join["trend"]==2) & (data_join["profile_type"]==False) &
-             (data_join["twitter_years"]==1) & (data_join["max_tw_day"]>180) &
+             (data_join["twitter_years"]==1) & (data_join["max_tw_day"]>210) &
               (data_join["evaluate_NLP"]=="No")] = "NLP-F5"                     
 
     #7.Possible fake behaviour profile in the last days
@@ -908,11 +917,11 @@ def contextual_analysis_profile(data, keyword_corpus, hashtag_corpus):
       data["positive_opinion"] = 0
     else:
       data["per_hash_key"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==1]) / len(tweets_df))
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]>=0.6)]= 2
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]<0.6)]= 3
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < 0.1) & (tweets_df["polarity"] > -0.1)]= 4
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]<0.6)]= 5
-      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]>=0.6)]= 6
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]>=0.6)]= 2 #negative opinion
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] <= -0.1) & (tweets_df["subjectivity"]<0.6)]= 3 # negative fact
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < 0.1) & (tweets_df["polarity"] > -0.1)]= 4 #neutral
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]<=0.5)]= 5 #positive fact
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] >= 0.1) & (tweets_df["subjectivity"]>0.5)]= 6 #positive opinion
       if len(tweets_df[tweets_df["terrorist_suggestion"]==2]) == 0:
         data["negative_opinion"] = 0
       else:
@@ -1076,39 +1085,50 @@ def define_hashtags(data):
 		hashtags.append(t)
 	data['hashtags']=hashtags
 	return data
-	
-def keyword_score(data,sample_data_tokenized, keyword_corpus):
-	scores = []
-	for i in range(len(sample_data_tokenized)):
-		score=0
-		for c in keyword_corpus.keywords:
-			for t in sample_data_tokenized[i]:
-				if c==t:
-					score+=1
-		try:
-			scores.append(score/len(sample_data_tokenized[i]))
-		except ZeroDivisionError:
-			scores.append(0)
-	data['keyword_score']=scores
-	return data
-
 
 def hashtag_score(data, hashtags_corpus):
-	scores=[]
-	data = define_hashtags(data)
-	for hshtgs in data.hashtags:
-		score = 0
-		for h in hshtgs:
-			#print(h)
-			for H in hashtags_corpus.hashtags:
-				if h == H:
-					score += 1
-		scores.append(score)
-	data['hashtag_score'] = scores
-	data['index'] = [i for i in range(len(data))]
-	data['names']= ['TWEET '+str(i) for i in range(len(data))]
-	return data
+    scores=[]
+    hashtags = []
+    data = define_hashtags(data)
+    for hshtgs in data.hashtags:
+      score = 0
+      hashtag = []
+      for h in hshtgs:
+        #print(h)
+        for H in hashtags_corpus.hashtags:
+          if h == H:
+            score += 1
+            hashtag.append(h)
+      hashtags.append(hashtag)      
+      scores.append(score)
+    data['hashtag_score'] = scores
+    data["terrorist_hashtags"] = hashtags
+    data['index'] = [i for i in range(len(data))]
+    data['names']= ['TWEET '+str(i) for i in range(len(data))]
+    return data
 	
+def keyword_score(data,sample_data_tokenized, keyword_corpus):
+    scores = []
+    words_list= []
+    for i in range(len(sample_data_tokenized)):
+      score=0
+      words = []
+      for c in keyword_corpus.keywords:
+        for t in sample_data_tokenized[i]:         
+          if c==t:
+            words.append(c)
+            score+=1
+      try:
+        scores.append(score/len(sample_data_tokenized[i]))
+        words_list.append(words)
+      except ZeroDivisionError:
+        scores.append(0)
+        words_list.append(0)
+    data["keyword_match"] = words_list
+    data['keyword_score']=scores
+    return data
+
+
 def terrorist_keywords_belonging(data):
 	temp = data[data['keyword_score']>0.3].reset_index(drop=True)
 	# Draw plot
@@ -1129,7 +1149,7 @@ def terrorist_keywords_belonging(data):
 	
 def pie_chart_keywords_terrorist(data):
 	plt.rcParams['figure.figsize'] = [9, 6]
-	data_terr = data[data['keyword_score']>0.2].reset_index(drop=True)
+	data_terr = data[data['keyword_score']>0.3].reset_index(drop=True)
 	p=0
 	n=0
 	o=0
